from django.core.management.base import BaseCommand
from django.db import transaction

from src.management.commands.default import delete_graphs, count_change
from src.management.commands.core import create_arc, get_arc
from src.models import Graph, Vertex, Arc


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('min_n', type=int)
        parser.add_argument('max_n', type=int)
        parser.add_argument('min_m', type=int)
        parser.add_argument('max_m', type=int)

    def handle(self, *args, **options):
        delete_graphs("all")

        min_n = options['min_n']
        max_n = options['max_n']
        min_m = options['min_m']
        max_m = options['max_m']
        for n in range(min_n, max_n + 1):
            for m in range(min_m, max_m + 1):
                self.create_tadpole(n, m)

    def create_tadpole(self, n, m):
        with transaction.atomic():
            for i in range(1, (2 ** n)):
                for j in range(1, (2 ** m)):
                    binary_number_n = bin(i)[2:].zfill(n)
                    binary_number_m = bin(j)[2:].zfill(m)

                    print(binary_number_n, binary_number_m)
                    graph = self.create_graph(binary_number_n, binary_number_m)
                    if graph:
                        graph.update()

    def create_graph(self, binary_number_n, binary_number_m):
        length = len(binary_number_n+binary_number_m)
        graph = Graph.objects.create(
            graph_type=Graph.TADPOLE,
            binary=binary_number_n + binary_number_m,
        )

        a = count_change(binary_number_n+binary_number_n[0]) + count_change(binary_number_m)
        if binary_number_n[0] == binary_number_m[0] and binary_number_n[-1] != binary_number_m[0]:
            return self.max_degree_3(binary_number_n, binary_number_m, graph, length, a)
        else:
            return self.max_degree_2()

    @staticmethod
    def max_degree_3(binary_number_n, binary_number_m, graph, length, a):
        arc_num = 1
        labeling = [i for i in range(1, length + 1)]
        label = length - a + 1
        labeling.remove(label)
        prev_vertex = vertex3 = Vertex.objects.create(graph=graph, label=label)
        for j, bit in enumerate(binary_number_n):
            vertex = vertex3 if j == len(binary_number_n) - 1 else Vertex.objects.create(graph=graph)
            arc_num = create_arc(
                vertex_in=prev_vertex if bit == '0' else vertex,
                vertex_out=vertex if bit == '0' else prev_vertex,
                arc_num=arc_num,
                label=arc_num + length
            )
            prev_vertex = vertex

        prev_vertex = vertex3
        for j, bit in enumerate(binary_number_m):
            vertex = Vertex.objects.create(graph=graph)
            arc_num = create_arc(
                vertex_in=prev_vertex if bit == '0' else vertex,
                vertex_out=vertex if bit == '0' else prev_vertex,
                arc_num=arc_num,
                label=arc_num + length
            )
            prev_vertex = vertex

        vertices = Vertex.objects.filter(graph=graph)
        for vertex in vertices:
            if vertex.max_degree == 2:
                label = min([x for x in labeling if x > (length - a + 1)])
                labeling.remove(label)
                vertex.label = label
                vertex.save()
            if vertex.max_degree == 1:
                label = min([x for x in labeling if x < (length - a + 1)])
                labeling.remove(label)
                vertex.label = label
                vertex.save()
        return graph

    @staticmethod
    def max_degree_2():
        return None

