from django.core.management.base import BaseCommand

import numpy as np
import networkx as nx
import matplotlib.pyplot as plt
from src.models import Graph, Vertex, Arc


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('id', type=int)
        # parser.add_argument('graph_type', type=str)
        # parser.add_argument('binary', type=str)

    def handle(self, *args, **options):
        id = options['id']
        # graph_type = options['graph_type']
        # binary = options['binary']

        graph = Graph.objects.filter(
            # graph_type=graph_type, binary=binary
            id=id
        ).first()

        if not graph:
            return

        vertices = Vertex.objects.filter(graph=graph)
        arcs = Arc.objects.filter(vertex_in__graph=graph)

        adjacency_matrix = np.zeros((len(vertices), len(vertices)))

        for arc in arcs:
            adjacency_matrix[arc.vertex_in.index - 1][arc.vertex_out.index - 1] = 1
            adjacency_matrix[arc.vertex_out.index - 1][arc.vertex_in.index - 1] = 1

        # Create a degree matrix
        degree_matrix = np.diag(np.sum(adjacency_matrix, axis=1))

        # Create Laplacian matrix
        laplacian_matrix = degree_matrix - adjacency_matrix

        # Calculate eigenvalues and eigenvectors of the Laplacian matrix
        eigenvalues, eigenvectors = np.linalg.eigh(laplacian_matrix)

        # Get the second smallest eigenvector (the smallest is constant)
        second_smallest_eigenvector = eigenvectors[:, 1]

        # Use the eigenvector values as x, y coordinates for visualization
        x_coordinates = second_smallest_eigenvector.real
        y_coordinates = eigenvectors[:, 2].real  # You can choose a different eigenvector if needed

        # Visualize the graph using matplotlib
        plt.scatter(x_coordinates, y_coordinates, c='blue', s=100)
        for i, vertex in enumerate(vertices):
            plt.text(x_coordinates[i], y_coordinates[i], str(vertex.index))

        # Draw lines between connected vertices
        for arc in arcs:
            index_in = arc.vertex_in.index - 1
            index_out = arc.vertex_out.index - 1
            plt.plot([x_coordinates[index_in], x_coordinates[index_out]],
                     [y_coordinates[index_in], y_coordinates[index_out]],
                     color='gray', linestyle='-', linewidth=1)

        plt.title('Graph Visualization with Edges using Laplacian Eigenmaps')
        plt.show()